#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE

# Copyright (C) 2013 National University of Defense Technology(NUDT) & Kylin Ltd

# Author:     
#     kobe Lee <xiangli@ubuntukylin.com>
#     maclin <majun@ubuntukylin.com>
# Maintainer:
#     maclin <majun@ubuntukylin.com>

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

### END LICENSE

import apt
import os
import aptsources.sourceslist
import apt.progress.base as apb
from apt.debfile import DebPackage
from apt.cache import FetchFailedException
import socket
socket.setdefaulttimeout(30)

import locale

#apt安装和更新缺少依赖时apt不会返回异常，且会继续触发安装的回调函数，人为处理异常，FLAG做为全局标识
FLAG = 1

# application actions, should sync with definition in models.enums
class AppActions:
    INSTALLDEPS = "install_deps"
    INSTALLDEBFILE = "install_debfile"
    INSTALL = "install"
    REMOVE = "remove"
    UPGRADE = "upgrade"
    CANCEL = "cancel"
    APPLY = "apply_changes"
    PURCHASE = "purchase"
    UPDATE = "update"
    UPDATE_FIRST = "update_first"
    ADD_SOURCE = "add_source"
    REMOVE_SOURCE = "remove_source"
    GET_SOURCES = "get_sources"

class FetchProcess(apb.AcquireProgress):
    '''Fetch Process'''
    def __init__(self, dbus_service, appname, action):
        apb.AcquireProgress.__init__(self)
        self.dbus_service = dbus_service
        self.appname = appname
        self.action = action

    def done(self, item):
#        print '#######all items download finished',item
#        if item is not None:
#            print "FetchProcess, done, Item:", self.appname,item.shortdesc, item.uri, item.owner
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.software_fetch_signal("down_done", kwarg)

    def fail(self, item):
#        print 'download failed'
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }
#        print "$$$$fectchprocess####fail:",kwarg
        self.dbus_service.software_fetch_signal("down_fail", kwarg)

    def fetch(self, item):
#        print 'one item download finished:',item
#        if item is not None:
        #print "FetchProcess, fetch, Item:", self.appname, item.shortdesc, item.uri, item.owner
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 "shortdesc":item.shortdesc,
                 "uri":item.uri,
                 }

        self.dbus_service.software_fetch_signal("down_fetch", kwarg)

    def ims_hit(self, item):
#        print 'ims_hit'
        pass

    def media_change(self, media, drive):
#        print 'media_change'
        pass

    def pulse(self, owner):

        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "download_bytes":str(self.current_bytes),
                 "total_bytes":str(self.total_bytes),
                 "download_items":str(self.current_items),
                 "total_items":str(self.total_items),
                 "action":str(self.action),
                 }

        if self.action == AppActions.UPDATE or self.action == AppActions.UPDATE_FIRST:
            if self.total_items!= 0:
                percent = float(self.current_items * 100.0 / self.total_items)
                if percent > self.percent:
                    self.percent = percent
#                self.percent = float(self.current_items * 100.0 / self.total_items)
        else:
            if self.total_bytes != 0:
                self.percent = float(self.current_bytes * 100.0 / self.total_bytes)

        self.dbus_service.software_fetch_signal("down_pulse",kwarg)

        # cancel the operation
        # if self.dbus_service.check_cancel_worker_item(self.appname) is True:
        #     self.dbus_service.software_fetch_signal("down_cancel",kwarg)
        #     return False

    def start(self):
        # Reset all our values.
        global FLAG
        FLAG = 1
        self.current_bytes = 0.0
        self.current_cps = 0.0
        self.current_items = 0
        self.elapsed_time = 0
        self.fetched_bytes = 0.0
        self.last_bytes = 0.0
        self.total_bytes = 0.0
        self.total_items = 0
        self.percent = 0
#        print 'fetch progress start ...',self.appname
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }
        self.dbus_service.software_fetch_signal("down_start", kwarg)

    def stop(self):
        global FLAG
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(200),
                 "action":str(self.action),
                 }
        if hasattr(self, "percent"):
            kwarg["download_percent"] = str(self.percent)
        else:
            kwarg["download_percent"] = str(100)
        if self.action == "update" or self.action == "update_first":
            self.dbus_service.set_uksc_not_working()
            if self.total_items > 0 and (self.current_items / self.total_items) < 1:
                kwarg["download_percent"] = str(-15)
        #apt安装和更新缺少依赖时apt不会返回异常，人为处理异常，通过下载停止来判断
        #if kwarg["download_percent"] == '0' and (self.action == "install" or self.action == "upgrade" or self.action == "install_deps"):
        #    kwarg["download_percent"] = str(-99)
        #    FLAG = 0
        #    self.dbus_service.set_uksc_not_working()
        self.dbus_service.software_fetch_signal("down_stop", kwarg)


class AptProcess(apb.InstallProgress):
    '''Apt progress'''
    def __init__(self, dbus_service, appname, action):
        apb.InstallProgress.__init__(self)
        self.dbus_service = dbus_service
        self.appname = appname
        self.percent = 0
        self.action = action

    def conffile(self, current, new):
#        print 'there is a conffile question'
        pass

    def error(self, pkg, errormsg):
#        print "AptProcess, error:", self.appname, pkg, errormsg
        global FLAG
        kwarg = {"apt_appname":self.appname,
                 "apt_percent":str(self.percent),
                 "action":str(self.action),
                 }
        self.dbus_service.set_uksc_not_working()
        if FLAG:
            self.dbus_service.software_apt_signal("apt_error", kwarg)
        else:
            FLAG =1

    def start_update(self):
#        print 'apt process start work', self.appname
        kwarg = {"apt_appname":self.appname,
                 "apt_percent":str(self.percent),
                 "action":str(self.action),
                 }
        if(self.action == AppActions.INSTALLDEBFILE):
            kwarg["apt_percent"] = "50"
        if FLAG:
            self.dbus_service.software_apt_signal("apt_start", kwarg)

    def finish_update(self):
        global FLAG
#        print 'apt process finished', self.appname
        kwarg = {"apt_appname":self.appname,
                 "apt_percent":str(200),
                 "action":str(self.action),
                 }
        if FLAG:
            self.dbus_service.software_apt_signal("apt_finish", kwarg)
        else:
            FLAG = 1
        #if self.appname == "kylin-software-center" and self.action == "upgrade":
        #    pass
        #else:
        self.dbus_service.set_uksc_not_working()

    def status_change(self, pkg, percent, status):
#        print "status_change:", self.appname, pkg
#        print str(int(percent)) + "%  status : " + status
#        self.percent = percent
#        if percent != self.percent:
#            print "&&&&&&&&&&&&&&&&&&&:",self.percent
        kwarg = {"apt_appname":str(self.appname),
#                 "install_percent":self.percent,
                 "apt_percent":str(percent),
                 "status":str(status),
                 "action":str(self.action),
                 }

#        print "####status_change:", kwarg
        if FLAG:
            self.dbus_service.software_apt_signal("apt_pulse", kwarg)


#class AptDaemon(threading.Thread):
"""
error number:
1           本地cache不存在软件包
2           下载失败
3           dpkg锁被占用
4           本地deb包读取出错
5           本地deb包未知错误
6           安装本地deb包出错
7           在线安装出错（重复安装）
8           在线安装未知错误
9           升级出错（没有对应升级包）
10          升级未知错误
11          卸载出错（未安装）
12          卸载未知错误
13          更新未知错误
"""
class AptDaemon():

    def __init__(self, dbus_service):
        self.dbus_service = dbus_service
        locale.setlocale(locale.LC_ALL, "zh_CN.UTF-8")
        self.cache = apt.Cache()

    # get package by pkgName
    def get_pkg_by_name(self, pkgName):
#        print pkgName
        try:
            pkg = self.cache[pkgName]
        except KeyError:
            raise WorkitemError(1, "Package %s is not  available" % pkgName)
        else:
            if pkg.candidate is None:
                raise WorkitemError(1, "Package %s is not  available" % pkgName)
            else:
                return pkg

        # except Exception, e:
        #     print e
        #     return "ERROR"

    # install deb file
    def install_debfile(self, path, kwargs=None):
        if not os.path.isfile(path):
            raise WorkitemError(4, "%s is unreadable file" % path)
        try:
            debfile = DebPackage(path)
        except IOError:
            raise WorkitemError(4, "%s is unreadable file" % path)
        except Exception as e:
            raise WorkitemError(5, e)
        self.cache.open()
        pkgName = debfile._sections["Package"]
        debfile.check() #do debfile.check for the next to do debfile.missing_deps
        if 0 == len(debfile.missing_deps):
            # try:
            res = debfile.install()
            if res:
                raise WorkitemError(6, "package manager failed")
            else:
                kwarg = {"apt_appname":pkgName,
                         "apt_percent":str(200),
                         "action":str(AppActions.INSTALLDEBFILE),
                         }
                self.dbus_service.software_apt_signal("apt_finish", kwarg)
                if pkgName == "kylin-software-center" and AppActions.INSTALLDEBFILE == "upgrade":
                    pass
                else:
                    self.dbus_service.set_uksc_not_working()
        else:
            raise WorkitemError(16, "dependence not be satisfied")

    # install deps
    def install_deps(self, path, kwargs=None):
        debfile = DebPackage(path)
        pkgName = debfile._sections["Package"]
        self.cache.open()
        debfile.check()
        deps = debfile.missing_deps

        if(len(deps) > 0):
            for pkgn in deps:
                self.cache.open()
                pkg = self.get_pkg_by_name(pkgn)
                pkg.mark_install()
                try:
                    self.cache.commit(FetchProcess(self.dbus_service, pkgn, AppActions.INSTALLDEPS), AptProcess(self.dbus_service, pkgn, AppActions.INSTALLDEPS))
                except Exception as e:
                    raise WorkitemError(6, e)

    # install package
    def install(self, pkgName, kwargs=None):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        if pkg.is_installed == True and pkg.is_upgradable == False:
            raise WorkitemError(7, "Package %s  is installed" % pkgName)
        pkg.mark_install()

        try:
            os.environ["DEBIAN_FRONTEND"] = "noninteractive"
            self.cache.commit(FetchProcess(self.dbus_service,pkgName,AppActions.INSTALL), AptProcess(self.dbus_service,pkgName,AppActions.INSTALL))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(8, e)

    # update package
    def upgrade(self, pkgName, kwargs=None):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        if pkg.is_installed and not pkg.installed_files:
            raise WorkitemError(8, "Package %s isn't installed" % pkgName)
        if pkg.is_upgradable is False:
            raise WorkitemError(9, "Package %s can't be upgraded" % pkgName)
        pkg.mark_upgrade()
        try:
            self.cache.commit(FetchProcess(self.dbus_service,pkgName,AppActions.UPGRADE), AptProcess(self.dbus_service,pkgName,AppActions.UPGRADE))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(10, e)

    # uninstall package
    def remove(self, pkgName, kwargs=None):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        if pkg.is_installed is False:
            raise WorkitemError(11, "Package %s isn't installed" % pkgName)
        pkg.mark_delete()

        try:
            self.cache.commit(None, AptProcess(self.dbus_service,pkgName,AppActions.REMOVE))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(12, e)


    # apt-get update
    def update(self, taskName, kwargs=None):
        self.cache.open()
        # quiet = False
        # if kwargs is not None:
        #     quiet = int(kwargs["quiet"])

        try:
            # if quiet == True:
            #     self.cache.update()
            # else:
            self.cache.update(fetch_progress=FetchProcess(self.dbus_service,taskName,AppActions.UPDATE))
        except Exception as e:
            raise WorkitemError(13, e)
        else:
            self.cache.open()

    # apt-get update first launch os
    def update_first(self, taskName, kwargs=None):
        self.cache.open()
        # quiet = False
        # if kwargs is not None:
        #     quiet = int(kwargs["quiet"])

        try:
            # if quiet == True:
            #     self.cache.update()
            # else:
            self.cache.update(fetch_progress=FetchProcess(self.dbus_service,taskName,AppActions.UPDATE_FIRST))
        except Exception as e:
            raise WorkitemError(13, e)
        else:
            self.cache.open()

    # check package status by pkgName, i = installed u = can update n = notinstall
    def check_pkg_status(self, pkgName):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        if(pkg == "ERROR"):
            return "ERROR"
        if(pkg.is_installed):
            if(pkg.is_upgradable):
                return "u"
            else:
                return "i"
        else:
            return "n"

    # check packages status by pkgNameList, i = installed u = can update n = notinstall
    def check_pkgs_status(self, pkgNameList):
        self.cache.open()
        pkgStatusDict = {}
        for pkgName in pkgNameList:
            pkg = self.get_pkg_by_name(pkgName)
            if(pkg == "ERROR"):
                continue
            if(pkg.is_installed):
                if(pkg.is_upgradable):
                    pkgStatusDict[pkgName] = "u"
                else:
                    pkgStatusDict[pkgName] = "i"
            else:
                pkgStatusDict[pkgName] = "n"

        return pkgStatusDict

    # check packages status by pkgNameList, i = installed u = can update n = notinstall
    def check_pkgs_status_rtn_list(self, pkgNameList):
        self.cache.open()
        pkgStatusList = []
        for pkgName in pkgNameList:
            pkg = self.get_pkg_by_name(pkgName)
            if(pkg == "ERROR"):
                    continue
            if(pkg.is_installed):
                if(pkg.is_upgradable):
                    pkgStatusList.append(pkgName + ":u")
                else:
                    pkgStatusList.append(pkgName + ":i")
            else:
                pkgStatusList.append(pkgName + ":n")

        self.dbus_service.software_check_status_signal(pkgStatusList)
        #return pkgStatusList

    # get all source item in /etc/apt/sources.list
    def get_sources(self, except_ubuntu):
        slist = []
        source = aptsources.sourceslist.SourcesList()
        for one in source.list:
            if(one.disabled == False and one.type != ""):
                if except_ubuntu:
                    if one.str().find(".ubuntu.com/ubuntu") == -1:
                        slist.append(one.str())
                    else:
                        continue
                else:
                    slist.append(one.str())
        return slist

    # add source in /etc/apt/sources.list
    def add_source(self,text):
        source = aptsources.sourceslist.SourcesList()

        slist = text.split()
        if(len(slist) < 3): # wrong source text
            return '3'


        for item in source.list:
            itemflag = 0
            for itemslist in slist:
                if(item.str().find(itemslist) == -1):
                    itemflag = 1
            if itemflag == 0:
                return '2' # 已存在的source

        type = slist[0]
        uri = slist[1]
        dist = slist[2]
        comps = []
        for i in range(3, len(slist)):
            comps.append(slist[i])
        source.add(type, uri, dist, comps)
        source.save()

        return '1'

    # remove source from /etc/apt/sources.list
    def remove_source(self,text):
        source = aptsources.sourceslist.SourcesList()
        sources = source.list
        for item in sources:
            if(item.str().find(text) != -1):
                source.remove(item)
        source.save()

        return True

    # add ubuntukylin source in /etc/apt/sources.list
    def add_source_ubuntukylin(self):
        source = aptsources.sourceslist.SourcesList()
        for item in source.list:
            if(item.str().find("deb http://archive.ubuntukylin.com:10006/ubuntukylin") != -1):
                return

        source.add("deb", "http://archive.ubuntukylin.com:10006/ubuntukylin/", "trusty main", "")
        source.save()

    # remove ubuntukylin source in /etc/apt/sources.list
    def remove_source_ubuntukylin(self):
        source = aptsources.sourceslist.SourcesList()
        sources = source.list
        for item in sources:
            if(item.str().find("deb http://archive.ubuntukylin.com:10006/ubuntukylin") != -1):
                source.remove(item)
        source.save()

class WorkitemError(Exception):

    def __init__(self, errornum, details = ""):
        self.errornum = errornum
        self.details = details


if __name__ == "__main__":
    ad = AptDaemon(None)

# 	print ad.check_pkgs_status(["gedit", "cairo-dock", "unity"])
#	print ad.check_pkgs_status_rtn_list(["gedit", "cairo-dock", "unity", "haha", "hehe"])
# 	ad.update()
    ad.add_source_ubuntukylin()
# 	ad.remove_source_ubuntukylin()

    while True:
        print("\ninput your command: ")
        cmd = input()
        if cmd == "l":
            for name in ad.pkgNameList:
                print(name + "\n")
        elif cmd == "i":
            print("input pkgName to install: ")
            pkgName = input()
            ad.install_pkg(pkgName)
        elif cmd == "n":
            print("input pkgName to uninstall: ")
            pkgName = input()
            ad.uninstall_pkg(pkgName)
        elif cmd == "u":
            print("input pkgName to update: ")
            pkgName = input()
            ad.update_pkg(pkgName)
        elif cmd == "c":
            print("input pkgName to check status: ")
            pkgName = input()
            print(ad.check_pkg_status(pkgName))
        else:
            print("nothing...")

# 	print ad.get_pkg_by_name('gedit')
    # pnl = ad.getpkglist()
    # print len(pnl)
# 	name1 = ad.search_pkgs_name('wesnoth-1.10-core')
# 	print name1
    # print 'aaa' + str(1)
# 	ad.install_pkg(name1)
# 	ad.uninstall_pkg(name1)
    # p = ad.get_pkg_by_name(name1)
    # print p.id
    # c = AptCache()
    # c.hahaha()
    # print c.hahaha()
    # pkgs = []
    # ca = apt.Cache()
    # i = 0
    # for a in ca:
    # 	i += 1
    # 	pkgs.append(a.name)
            # print a.name
    # print i
    # nanop = ca['nano']
    # print nanop
    # nanop.mark_install()
    # cache.commit()




import os
import apt
import subprocess
from models.enums import UBUNTUKYLIN_CACHE_ICON_PATH,UBUNTUKYLIN_CACHE_ICON_PATH,UBUNTUKYLIN_RES_ICON_PATH,KYLIN_SYSTEM_ICON_48_PATH,UK_SYSTEM_ICON_48_PATH,KYLIN_SYSTEM_ICON_128_PATH,UK_SYSTEM_ICON_128_PATH,\
                         UBUNTUKYLIN_CACHE_COVERIMAGE_PATH,UBUNTUKYLIN_RES_COVERIMAGE_PATH,UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH,\
                         UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH,UBUNTUKYLIN_RES_AD_PATH,KYLIN_SYSTEM_ICON_64_PATH,KYLIN_SVG_ICON_PATH,UK_SVG_ICON_PATH,UK_SYSTEM_ICON_64_PATH,\
                         SVG_ICON_PATH,ICON_48_PATH,ICON_64_PATH,ICON_128_PATH,KYLIN_SYSTEM_ICON_24_PATH
from models.enums import UKSC_CACHE_DIR
from models.globals import Globals
from gi.repository import Gio

KSC_INSTALL_ICON_PATH = "/usr/share/kylin-software-center/data/icons/"


#
# 函数：获取图标地址返回给调用接口的对象
#
def get_icon_path(app_name):
    if(app_name == "pluma"):
        app_name = "accessories-text-editor"
    if (os.path.isfile(SVG_ICON_PATH + str(app_name) + ".svg")):
        return SVG_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".png")):
        return ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".jpg")):
        return ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".png")):
        return ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".jpg")):
        return ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".png")):
        return ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".jpg")):
        return ICON_48_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SVG_ICON_PATH + str(app_name) + ".svg")):
        return KYLIN_SVG_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(UK_SVG_ICON_PATH + str(app_name) + ".svg")):
        return UK_SVG_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(UK_SVG_ICON_PATH + str(app_name) + ".svg")):
        return UK_SVG_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
        return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
        return UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
        return UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
        return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
        return UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
        return UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
        return UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KSC_INSTALL_ICON_PATH + str(app_name) + ".png")):
        return KSC_INSTALL_ICON_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KSC_INSTALL_ICON_PATH + str(app_name) + ".jpg")):
        return KSC_INSTALL_ICON_PATH + str(app_name) + ".jpg"
    else:
        return KSC_INSTALL_ICON_PATH + "default.png"

#
# 函数：获取图标地址的处理函数
#
def get_small_icon_path(app_name):
    if (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
        return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
        return Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
        return Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
        return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
        return Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
        return Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
        return Globals.UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(SVG_ICON_PATH + str(app_name) + ".svg")):
        return SVG_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".png")):
        return ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".jpg")):
        return ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".png")):
        return ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".jpg")):
        return ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".png")):
        return ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".jpg")):
        return ICON_48_PATH + str(app_name) + ".jpg"
    else:
        return UBUNTUKYLIN_RES_ICON_PATH + "default.png"

def get_title_icon(app_name):
    if (os.path.isfile(KYLIN_SYSTEM_ICON_24_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_24_PATH + str(app_name) + ".jpg")):
        return KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_24_PATH + str(app_name) + ".png")):
        return Globals.UK_SYSTEM_ICON_24_PATH + str(app_name) + ".png"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_24_PATH + str(app_name) + ".jpg")):
        return Globals.UK_SYSTEM_ICON_24_PATH + str(app_name) + ".jpg"


#
# 函数：获取图标地址的处理函数
#
def get_big_icon_path(app_name):
    if(app_name == "pluma"):
        app_name = "accessories-text-editor"
    if(app_name == Globals.APPNAME):
        if (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
            return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
        elif (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
            return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
            return Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
        elif (os.path.isfile(Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
            return Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
            return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
        elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
            return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
            return Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
        elif (os.path.isfile(Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
            return Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
            return KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
        elif (os.path.isfile(Globals.UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
            return Globals.UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
        elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png")):
            return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png"
        elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg")):
            return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".png")):
            return UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".png"
        elif (os.path.isfile(UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".jpg")):
            return UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(SVG_ICON_PATH + str(app_name) + ".svg")):
            return SVG_ICON_PATH + str(app_name) + ".svg"
        elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".png")):
            return ICON_128_PATH + str(app_name) + ".png"
        elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".jpg")):
            return ICON_128_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".png")):
            return ICON_64_PATH + str(app_name) + ".png"
        elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".jpg")):
            return ICON_64_PATH + str(app_name) + ".jpg"
        elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".png")):
            return ICON_48_PATH + str(app_name) + ".png"
        elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".jpg")):
            return ICON_48_PATH + str(app_name) + ".jpg"
        else:
            return UBUNTUKYLIN_RES_ICON_PATH + "default.png"
    if (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
        return KYLIN_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png")):
        return Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg")):
        return Globals.UK_SYSTEM_ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
        return KYLIN_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png")):
        return Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg")):
        return Globals.UK_SYSTEM_ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
        return KYLIN_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(Globals.UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png")):
        return Globals.UK_SYSTEM_ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".svg")):
        return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_CACHE_ICON_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".svg")):
        return UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_RES_ICON_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(SVG_ICON_PATH + str(app_name) + ".svg")):
        return SVG_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".png")):
        return ICON_128_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_128_PATH + str(app_name) + ".jpg")):
        return ICON_128_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".png")):
        return ICON_64_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_64_PATH + str(app_name) + ".jpg")):
        return ICON_64_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".png")):
        return ICON_48_PATH + str(app_name) + ".png"
    elif (os.path.isfile(ICON_48_PATH + str(app_name) + ".jpg")):
        return ICON_48_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(KYLIN_SVG_ICON_PATH + str(app_name) + ".svg")):
        return KYLIN_SVG_ICON_PATH + str(app_name) + ".svg"
    elif (os.path.isfile(UK_SVG_ICON_PATH + str(app_name) + ".svg")):
        return UK_SVG_ICON_PATH + str(app_name) + ".svg"
    else:
        return UBUNTUKYLIN_RES_ICON_PATH + "default.png"

#
#函数：获取封面图地址处理函数
#
def get_coverimage_path(app_name):
    if(os.path.isfile(UBUNTUKYLIN_CACHE_COVERIMAGE_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_CACHE_COVERIMAGE_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_COVERIMAGE_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_CACHE_COVERIMAGE_PATH + str(app_name) + ".jpg"
    elif(os.path.isfile(UBUNTUKYLIN_RES_COVERIMAGE_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_RES_COVERIMAGE_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_RES_COVERIMAGE_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_RES_COVERIMAGE_PATH + str(app_name) + ".jpg"
    else:
        return UBUNTUKYLIN_RES_COVERIMAGE_PATH + "default.png"

#
#函数：获取封面图地址处理函数
#
def get_coverimage(icon_path):
    if(os.path.isfile(UBUNTUKYLIN_CACHE_COVERIMAGE_PATH + icon_path)):
        return UBUNTUKYLIN_CACHE_COVERIMAGE_PATH + icon_path
    elif (os.path.isfile(UBUNTUKYLIN_RES_COVERIMAGE_PATH + icon_path)):
        return UBUNTUKYLIN_RES_COVERIMAGE_PATH + icon_path
    else:
        return UBUNTUKYLIN_RES_COVERIMAGE_PATH + "default.png"

#
#函数：获取新品上架图
#
def get_new_arrivals_image_path(app_name):
    if (os.path.isfile(UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH + str(app_name) + ".jpg"
    elif (os.path.isfile(UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH + str(app_name) + ".png")):
        return UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH + str(app_name) + ".png"
    elif (os.path.isfile(UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH + str(app_name) + ".jpg")):
        return UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH + str(app_name) + ".jpg"
    else:
        return UBUNTUKYLIN_RES_COVERIMAGE_PATH + "default.png"

#
#函数：获取新品上架图
#
def get_new_arrivals_image(path):
    if (os.path.isfile(UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH + path)):
        return UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH + path
    elif (os.path.isfile(UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH + path)):
        return UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH + path
    else:
        return UBUNTUKYLIN_RES_COVERIMAGE_PATH + "default.png"

def get_ads_image(file_name):
    if (os.path.isfile(UBUNTUKYLIN_RES_AD_PATH + file_name)):
        return UBUNTUKYLIN_RES_AD_PATH + file_name
    else:
        return None

#
# 函数名: 通过默认浏览器打开链接
# Function: Open Url with default browser
#
def webbrowser_open_url(url):
    if(url == None):
        url = "https://www.kylinos.cn/"
    browser_name = "qaxbrowser-safe-stable"
    apt_ca = apt.Cache()
    try:
        try:
            appList = Gio.AppInfo.get_default_for_type("x-scheme-handler/https", False)
            name = Gio.AppInfo.get_name(appList)
            executable = Gio.AppInfo.get_executable(appList)
            os.system(executable + " " + url + " & > /dev/null")
            return
        except:
            pass
        try:
            config_home = UKSC_CACHE_DIR.rsplit("/", 2)[0] + "/.config/"
            mimeapps_path = config_home + "mimeapps.list"
            with open(mimeapps_path, "r", encoding="utf-8") as mine_file:
                for line in mine_file.readlines():
                    if line.find("x-scheme-handler/https") != -1:
                        browser_name = (line.rsplit("=", 1)[1]).rsplit(".", 1)[0]
                        if Globals.DEBUG_SWITCH:
                            print("broswer name: %s" % browser_name)
                        break
            if(browser_name == "browser360-cn"):
                browser_name = "browser360-cn-stable"
            elif(browser_name == "qaxbrowser-safe"):
                browser_name = "qaxbrowser-safe-stable"
            try:
                pkg = apt_ca[browser_name]
                if (pkg.is_installed == True):
                    os.system(browser_name + " " + url + " & > /dev/null")
                    return
            except:
                pass
        except:
            pass
        try:
            browser_name = "qaxbrowser-safe-stable"
            pkg = apt_ca[browser_name]
            if(pkg.is_installed == True):
                os.system(browser_name + " " + url + " & > /dev/null")
            else:
                browser_name = "firefox"
                try:
                    pkg = apt_ca[browser_name]
                    if (pkg.is_installed == True):
                        os.system(browser_name + " " + url + " & > /dev/null")
                    else:
                        try:
                            browser_name = "firefox-esr"
                            pkg = apt_ca[browser_name]
                            if (pkg.is_installed == True):
                                os.system(browser_name + " " + url + " & > /dev/null")
                        except:
                            browser_name = "lbrowser"
                            try:
                                pkg = apt_ca[browser_name]
                                if (pkg.is_installed == True):
                                    os.system(browser_name + " " + url + " & > /dev/null")
                            except:
                                pass
                except:
                    try:
                        browser_name = "firefox-esr"
                        pkg = apt_ca[browser_name]
                        os.system(browser_name + " " + url + " & > /dev/null")
                    except:
                        browser_name = "lbrowser"
                        try:
                            pkg = apt_ca[browser_name]
                            if (pkg.is_installed == True):
                                os.system(browser_name + " " + url + " & > /dev/null")
                        except:
                            pass
        except:
            browser_name = "firefox"
            try:
                pkg = apt_ca[browser_name]
                if(pkg.is_installed == True):
                    os.system(browser_name + " " + url + " & > /dev/null")
                else:
                    try:
                        browser_name = "firefox-esr"
                        pkg = apt_ca[browser_name]
                        if(pkg.is_installed):
                            os.system(browser_name + " " + url + " & > /dev/null")
                    except:
                        browser_name = "lbrowser"
                        try:
                            pkg = apt_ca[browser_name]
                            if (pkg.is_installed == True):
                                os.system(browser_name + " " + url + " & > /dev/null")
                        except:
                            pass
            except:
                try:
                    browser_name = "firefox-esr"
                    pkg = apt_ca[browser_name]
                    if(pkg.is_installed):
                        os.system(browser_name + " " + url + " & > /dev/null")
                    else:
                        browser_name = "lbrowser"
                        try:
                            pkg = apt_ca[browser_name]
                            if (pkg.is_installed == True):
                                os.system(browser_name + " " + url + " & > /dev/null")
                        except:
                            pass
                except:
                    browser_name = "lbrowser"
                    try:
                        pkg = apt_ca[browser_name]
                        if (pkg.is_installed == True):
                            os.system(browser_name + " " + url + " & > /dev/null")
                    except:
                        pass
    except Exception as e:
        pass


def get_desktop_dict(system_version):
    if system_version == "V10":
        dingtalk = "DingTalkLinux"
    else:
        dingtalk = "nwjs-dingtalk"
    # moshengren add
    PKG_NAME = {
        "wps-office": "wps-office-wps",
        "uget": "uget-gtk",
        "eclipse-platform": "eclipse",
        "software-center": "ubuntu-software-center",
        "mathwar": "MathWar",
        "gnome-disk-utility": "gnome-disks",
        "kino": "Kino",
        "monajat-applet": "monajat",
        "system-config-printer-applet": "system-config-printer",
        "xterm": "debian-uxterm",
        "virtualbox-qt": "virtualbox",
        "lovewallpaper": "love-wallpaper",
        "steam-launcher": "steam",
        "obs-studio": "obs",
        "google-chrome-stable": "google-chrome",
        "youker-assistant": "kylin-assistant",
        "crossover:i386": "/opt/cxoffice/bin/crossover",  # crossover:i386
        "gnome-screenshot": "org.gnome.Screenshot",
        # "gnome-mines":  "gnomine",
        "qtcreator": "org.qt-project.qtcreator",
        "suwellreaderpro": "SuwellReader",
        "qaxbrowser-safe-stable": "qaxbrowser-safe",
        "foxitofficesuite": "FoxitofficeSuite",
        "linkdoodsetup": "Linkdood",
        "yozo-office": "yozo-writer",
        "zwcad-linuxpreinst": "ZWCAD-LinuxPreInst",
        "browser360-cn-stable": "browser360-cn",
        "360safe": "start360safe",
        "eleanscan": "Elean",
        "jingyunsd": "JingyunSd",
        "ukui-biometric-manager": "biometric-manager",
        "linuxqq": "qq",
        "tenvideo-universal": "TencentVideo",
        "teamviewer": "com.teamviewer.TeamViewer",
        "retext": "me.mitya57.ReText",
        "youku-app": "YouKu",
        "zjcakeymanager": "keymanager",
        "foundercebreader": "CEBReader",
        "ofdreader": "OFDReader",
        "founderdoceditor": "DocEditor",
        "qaxbrowser-pioneer-stable": "qaxbrowser-pioneer",
        "dianjureader": "DianjuReader",
        "cn.lanxin": "lanxin",
        "parole": "org.xfce.Parole",
        "isignaturechina": "isignature",
        "ffado-mixer-qt4": "ffado.org-ffadomixer",
        "umbrello": "org.kde.umbrello",
        "kig": "org.kde.kig",
        "krdc": "org.kde.krdc",
        "kexi": "org.kde.kexi-3.1",
        "krusader": "org.kde.krusader",
        "kbruch": "org.kde.kbruch",
        "calligrawords": "org.kde.calligrawords",
        "zescrow-client": "zEscrow",
        "okular": "org.kde.okular",
        "digikam": "org.kde.digikam",
        "acetoneiso": "AcetoneISO",
        "k3b": "org.kde.k3b",
        "showfoto": "org.kde.showfoto",
        "skanlite": "org.kde.skanlite",
        "kcachegrind": "org.kde.kcachegrind",
        "baobab": "org.gnome.baobab",
        "aisleriot": "sol",
        "sofa-apps": "sofa",
        "kubrick": "org.kde.kubrick",
        "kwrite": "org.kde.kwrite",
        "gupnp-tools": "gupnp-av-cp",
        "kturtle": "org.kde.kturtle",
        "vino": "vino-server",
        "yakuake": "org.kde.yakuake",
        "kompare": "org.kde.kompare",
        # "ktouch":"rg.kde.ktouch.desktop",
        "freeciv-client-gtk": "freeciv-gtk2",
        "owncloud-client": "owncloud",
        "qtsmbstatus-light": "qtsmbstatusl",
        "supertux": "supertux2",
        "kylin-log-viewer": "logview",
        "iagno": "org.gnome.Reversi",
        "kile": "org.kde.kile",
        "meld": "org.gnome.meld",
        "bpython": "org.bpython-interpreter.bpython",
        "spyder": "spyder3",
        "kdiff3": "org.kde.kdiff3",
        "poedit": "net.poedit.Poedit",
        "ghex": "org.gnome.GHex",
        "kdevelop": "org.kde.kdevelop",
        "d-feet": "org.gnome.dfeet",
        "gwenview": "org.kde.gwenview",
        "gthumb": "org.gnome.gThumb",
        "quodlibet": "io.github.quodlibet.QuodLibet",
        "lingot": "org.nongnu.lingot",
        "gnome-subtitles": "org.gnome.GnomeSubtitles",
        "devede": "devede_ng.py",
        "sound-juicer": "org.gnome.SoundJuicer",
        "goobox": "org.gnome.Goobox",
        "rosegarden": "com.rosegardenmusic.rosegarden",
        "songwrite": "songwrite3",
        "qbittorrent": "org.qbittorrent.qBittorrent",
        "remmina": "org.remmina.Remmina",
        "midori": "org.midori_browser.Midori",
        "geary": "org.gnome.Geary",
        "ktouch": "org.kde.ktouch",
        "wordnet-gui": "wordnet",
        "wxmaxima": "io.github.wxmaxima_developers.wxMaxima",
        "file-roller": "org.gnome.FileRoller",
        "rsibreak": "org.kde.rsibreak",
        "catfish": "org.xfce.Catfish",
        "sweeper": "org.kde.sweeper",
        "blueman": "blueman-manager",
        "nautilus": "org.gnome.Nautilus",
        "bleachbit": "org.bleachbit.BleachBit",
        "meritous": "net.asceai.meritous",
        "lightsoff": "org.gnome.LightsOff",
        "gnome-mines": "org.gnome.Mines",
        "gnome-chess": "org.gnome.Chess",
        "gnome-tetravex": "org.gnome.Tetravex",
        "kylin-burner": "org.kde.kylin-burner",
        "recoll": "recoll-searchgui",
        "dingtalk": dingtalk

    }

    Globals.PKG_NAME = PKG_NAME



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5 import QtCore, QtGui
from ui.starwidget import StarWidget
from ui.messagebox import MessageBox
from models.enums import Signals

# from ui.loadingdiv import *
import sip
import math
import gettext

gettext.bindtextdomain("ubuntu-kylin-software-center", "/usr/share/locale")
gettext.textdomain("ubuntu-kylin-software-center")
_ = gettext.gettext
from models.application import *
from models.enums import setLongTextToElideFormat


# from models.enums import CACHE_DIR,BUTTON_PRESSED,BUTTON_ENTER,BUTTON_LEAVE,BUTTON_DISABLE,AppMsg
#


class Screenshot(QPushButton):
    def __init__(self,card_width,card_height,num,app_detail,parent=None):
        super(Screenshot, self).__init__(parent)
        self.card_width = card_width
        self.card_height = card_height
        self.num = num
        self.app_detail = app_detail
        self.resize(card_width, card_height)
        self.hide()
        self.screen_outer_frame = QLabel(self)
        self.screen_outer_frame.resize(self.width()-15*2, self.height()-15*2)
        # self.screen_outer_frame1.move(51, 79)
        self.screen_outer_frame.move(15,15)
        self.screen_outer_frame.setStyleSheet(
            "QLabel{border-radius: 0px;border-image: url(./res/screenshot_2.png) 0 0 0 0 stretch strectch;border-radius: 0px;background-color:transparent;}")
        self.screen_outer_frame.hide()
        self.screen_shot = QLabel(self)
        self.screen_shot.resize(self.screen_outer_frame.width()-12*2, self.screen_outer_frame.height()-12*2)
        self.screen_shot.move(self.screen_outer_frame.x() + 12, self.screen_outer_frame.y() + 12)
        # self.screen_shot1.move(58, 86)
        self.setFocusPolicy(Qt.NoFocus)

        self.screen_shot.hide()
        self.surface_color = QLabel(self)
        self.surface_color.resize(self.width(), 78)
        # self.surface_color1.move(0, 181)
        if (Globals.THEME_COLOR == 0):
            self.surface_color.setStyleSheet(
                "QLabel{border-image: url(./res/surface_color.png) 0 0 0 0 stretch strectch;border-radius: 0px;background-color:transparent;}")
        else:
            self.surface_color.setStyleSheet(
                "QLabel{border-image: url(./res/surface_color_dark.png) 0 0 0 0 stretch strectch;border-radius: 0px;background-color:transparent;}")

        # self.surface_color1.setStyleSheet("QLabel{background: qlineargradient(x1:0, y1:0, x2:0, y2:1, stop:0 rgba(247,251,251,0), stop:1 #3790FA);border-bottom-left-radius:12px;border-bottom-right-radius:12px;opacity: 0.49;}")
        self.surface_color.move(0, self.height() - self.surface_color.height())
        self.surface_color.hide()

        self.defualt_photo = QLabel(self)
        self.defualt_photo.setFixedSize(30, 30)
        self.defualt_photo.move(int((self.width() - self.defualt_photo.width()) / 2),
                                 int((self.height() - self.defualt_photo.height()) / 2))

        self.defualt_photo.setStyleSheet(
            "QLabel{background-image:url(./res/defualt_screen.png);border-radius: 0px;background-color:transparent;}")
        self.defualt_photo.hide()
        self.change_color()

    def change_color(self):
        if(Globals.THEME_COLOR == 0):
            self.setStyleSheet("QPushButton{background-color: #F4F4F4;border-radius: 20px;}")
            self.surface_color.setStyleSheet(
                "QLabel{border-image: url(./res/surface_color.png) 0 0 0 0 stretch strectch;border-radius: 0px;background-color:transparent;}")
        else:
            self.setStyleSheet(
                "QPushButton{background-color: rgba(61,61,65,0.5);border-radius: 20px;}")
            self.surface_color.setStyleSheet(
                "QLabel{border-image: url(./res/surface_color_dark.png) 0 0 0 0 stretch strectch;border-radius: 0px;background-color:transparent;}")

    def signal_connect(self):
        self.clicked.connect(lambda: self.app_detail.show_screen_detail(self.num))



class Shot_Detail(QFrame, Signals):
    ad_move_left_time = QTimer()
    ad_move_right_time = QTimer()
    final_x_position = 0
    clicked_order = {}
    shot_pic = []
    pic_order = 0
    max_pic_num = 20

    def __init__(self, shot_phone,w,y,app_detail, parent = None):
        QWidget.__init__(self,parent)
        self.resize(w,y)
        self.m_parent = parent
        self.app_detail = app_detail
        self.setStyleSheet("QFrame{background-color:rgba(0, 0, 0, 0.7);border-bottom-left-radius:6px;border-bottom-right-radius:6px; }")
        self.left_btn = QPushButton(self)
        self.left_btn.setFocusPolicy(Qt.NoFocus)
        self.left_btn.resize(50, 70)
        # self.i = 0
        self.left_btn.move(100, int((self.height() - self.left_btn.height()) / 2))
        # self.left_btn.hide()
        self.left_btn.setEnabled(False)

        self.right_btn = QPushButton(self)
        self.right_btn.setFocusPolicy(Qt.NoFocus)
        self.right_btn.resize(50, 70)
        self.right_btn.move(self.width()-self.right_btn.width()-100,int((self.height() - self.right_btn.height()) / 2))

        self.shot_pic = shot_phone
        screen_count = len(self.shot_pic)
        # self.app_detail.screen_shot1.show()
        self.photo_widget = QWidget(self)
        self.photo_widget.setStyleSheet("QWidget{border:1px ;border-radius:12px;background:transparent;}")
        self.photo_widget.move(self.left_btn.x()+self.left_btn.width()+20, 76)
        self.photo_widget.resize(self.width()-2*(self.left_btn.x()+self.left_btn.width()+20), self.height()-2*76)

        self.photo_widget_slide = QWidget(self.photo_widget)
        self.photo_widget_slide.setStyleSheet("QWidget{border:1px ;border-radius:12px;background:transparent;}")

        # self.photo_widget_slide.resize(1000, 680)
        self.close_btn = QPushButton(self)

        self.close_btn.setFocusPolicy(Qt.NoFocus)
        self.close_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:15px;image: url(./res/close_shot.png);}")

        self.close_btn.setFixedSize(40, 40)
        self.close_btn.move(self.width()-self.close_btn.width()-30,30)
        self.close_btn.clicked.connect(self.close_widget)

        self.save_pic = []
        for i in range(self.max_pic_num):
            self.photo = Photo("",self.photo_widget.width(),self.photo_widget.height(),self.photo_widget_slide)
            self.photo.move(self.photo_widget.width()*i, 0)
            self.save_pic.append(self.photo)

        # 更新截图
        self.shot_pic_change(self.shot_pic)

        # # 点击截图
        # self.shot_pic_click(picture_order)

        self.right_btn.clicked.connect(self.slot_click_right_btn)
        self.left_btn.clicked.connect(self.slot_click_left_btn)
        self.ad_move_left_time.timeout.connect(self.slot_slide_left)
        self.ad_move_right_time.timeout.connect(self.slot_slide_right)

    def re_move(self,shot_screen_list):
        self.left_btn.move(100, int((self.height() - self.left_btn.height()) / 2))
        self.right_btn.move(self.width()-self.right_btn.width()-100,int((self.height() - self.right_btn.height()) / 2))
        self.close_btn.move(self.width() - self.close_btn.width() - 30, 30)
        self.photo_widget.resize(self.width() - 2 * (self.left_btn.x() + self.left_btn.width() + 20),self.height() - 2 * 76)
        self.photo_widget.move(self.left_btn.x() + self.left_btn.width() + 20, 76)
        photos = self.photo_widget_slide.children()
        self.photo_widget_slide.resize(self.photo_widget.width() * len(photos), self.photo_widget.height())
        # self.photo_widget_slide.move(0, 0)
        num = 0
        for photo in photos:
            photo.resize(self.photo_widget.width(),self.photo_widget.height())
            photo.re_move()
            photo.move(self.photo_widget.width()*num,int((self.photo_widget_slide.height()-photo.height())/2))
            if(len(self.save_pic) >= num):
                photo.resize_image()
                # photo.shot_photo_change(shot_screen_list[num])
                num = num +1
        self.shot_pic_click(self.pic_order)
        # if(self.pic_order == 1):
        #     self.left_btn.setEnabled(False)
        #     self.left_btn.setStyleSheet(
        #         "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_false.png);}")
        #     self.right_btn.setEnabled(True)
        #     self.right_btn.setStyleSheet(
        #         "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}")
        # elif

    def close_widget(self):
        try:
            # self.i += 1
            # print(self.i)
            self.left_btn.setEnabled(True)
            self.right_btn.setEnabled(True)
            self.hide()
            self.m_parent.hide()
        except Exception:
            pass

    # 函数：点击右边按钮
    def slot_click_right_btn(self):

        if (len(self.shot_pic) > 1):

            self.pic_order = self.pic_order + 1
            self.left_btn.setEnabled(True)
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}")

            x = self.photo_widget_slide.x()
            if (self.final_x_position == 0):

                self.final_x_position =self.photo_widget_slide.x()-self.photo_widget.width()
                Globals.POS = self.final_x_position
                self.ad_move_left_time.start(1)
            else:

                self.final_x_position = self.photo_widget_slide.x() - self.photo_widget.width()
                Globals.POS = self.final_x_position
                self.ad_move_left_time.start(1)

    #
    # #
    # 函数：向左滑动:1.左边滑动。2.滑到终点按钮置灰。
    def slot_slide_left(self):
        self.final_x_position = Globals.POS
        self.right_btn.setEnabled(False)
        self.right_btn.setStyleSheet(
            "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_false.png);}")
        self.left_btn.setEnabled(False)
        self.left_btn.setStyleSheet(
            "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_false.png);}")

        x = self.photo_widget_slide.x()
        if(x-100 < self.final_x_position):
            self.photo_widget_slide.move(self.final_x_position, 0)
        else:
            self.photo_widget_slide.move(x - 100, 0)
        self.ad_move_left_time.start(1)
        x = self.photo_widget_slide.x()
        if (x % self.final_x_position ==0) :
            self.ad_move_left_time.stop()
            self.right_btn.setEnabled(True)
            self.right_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}")

            self.left_btn.setEnabled(True)
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}")

            # self.final_x_position = 0
            # if(len(self.shot_pic)>4):
            #     if (x == -self.photo_widget.width()*(4-1)):
            #
            #         self.right_btn.setEnabled(False)
            #         self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_false.png);}")
            # else:
            if (x == -self.photo_widget.width() * (len(self.shot_pic) - 1)):
                self.right_btn.setEnabled(False)
                self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_false.png);}")

    #
    # 函数：点击向左的箭头
    #
    def slot_click_left_btn(self):

        self.pic_order = self.pic_order -1
        self.right_btn.setEnabled(True)
        self.right_btn.setStyleSheet(
            "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}")

        x = self.photo_widget_slide.x()

        if (self.final_x_position == 0):
            self.final_x_position =  self.photo_widget_slide.x()+ self.photo_widget.width()
            Globals.POS = self.final_x_position
            self.ad_move_right_time.start(0.1)
        else:
            self.final_x_position = self.photo_widget_slide.x() + self.photo_widget.width()
            Globals.POS = self.final_x_position
            self.ad_move_right_time.start(0.1)

    #
    # 函数向右滑动

    def slot_slide_right(self):
        self.ad_move_left_time.stop()
        self.final_x_position = Globals.POS
        self.right_btn.setEnabled(False)
        self.right_btn.setStyleSheet(
            "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_false.png);}")
        self.left_btn.setEnabled(False)
        self.left_btn.setStyleSheet(
            "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_false.png);}")
        x = self.photo_widget_slide.x()
        if(x+100 > self.final_x_position):
            self.photo_widget_slide.move(self.final_x_position, 0)
        else:
            self.photo_widget_slide.move(x + 100, 0)
        self.ad_move_right_time.start(1)
        x = self.photo_widget_slide.x()
        if (x == self.final_x_position):
            self.ad_move_right_time.stop()
            self.left_btn.setEnabled(True)
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}")

            self.right_btn.setEnabled(True)
            self.right_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}")

            # self.final_x_position = 0
        if (x == 0):
            self.left_btn.setEnabled(False)
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_false.png);}")

    # 截图改变
    def shot_pic_change(self, shot_photo):
        self.shot_pic = shot_photo
        # if(range(len(self.shot_pic) >4)):
        #     range_num = 4
        # else:
        range_num = len(self.shot_pic)
        for i in range(range_num):
            self.save_pic[i].shot_photo_change(self.shot_pic[i])
            self.save_pic[i].show()

        for i in range(len(self.shot_pic), self.max_pic_num):
            self.save_pic[i].hide()

        self.photo_widget_slide.resize(self.photo_widget.width() * len(self.shot_pic), self.photo_widget.height())

    # 截图点击
    def shot_pic_click(self, pic_order):
        self.pic_order = pic_order
        self.app_detail.m_parent.mw.hide_tip_widget()
        if pic_order == 1:
            self.photo_widget_slide.move(0, 0)
            self.left_btn.setEnabled(False)
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_false.png);}")
            if len(self.shot_pic) >1:
                self.right_btn.setStyleSheet(
                    "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}")
            else:
                self.right_btn.setStyleSheet(
                    "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_false.png);}")

        elif pic_order == len(self.shot_pic):
            self.photo_widget_slide.move(-self.photo_widget.width() * (pic_order - 1), 0)
            self.left_btn.setEnabled(True)
            self.right_btn.setEnabled(False)
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}")
            self.right_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_false.png);}")

        else:
            self.photo_widget_slide.move(-self.photo_widget.width() * (pic_order - 1), 0)
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bleft_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bleft_clicked.png);}")
            if len(self.shot_pic) > 2:
                self.right_btn.setStyleSheet(
                    "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_default.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/Bright_clicked.png);}")
            else:
                self.right_btn.setStyleSheet(
                    "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/Bright_false.png);}")
            self.left_btn.setEnabled(True)
        self.photo_widget_slide.show()
        self.show()


class Photo(QWidget):

    def __init__(self, shot_photo,w,h, parent=None):
        QWidget.__init__(self,parent)
        self.Photo = QLabel(self)
        self.Photo.move(0,0)
        self.Photo.resize(w,h)
        self.photo = QLabel(self.Photo)
        self.photo.resize(w,h)
        self.photo.setStyleSheet("QLabel{border-radius: 8px;border-image: url('" + shot_photo+ "') 0 0 0 0 stretch strectch;background:transparent;}")
        self.photo_path = ""
        self.photo_image = None

    def re_move(self):
        self.Photo.resize(self.width(), self.height())

    def resize_image(self):
        if(self.photo_image != None):
            if self.photo_image.width() > self.Photo.width() or self.photo_image.height() > self.Photo.height():

                if self.photo_image.width()/self.Photo.width()  > self.photo_image.height()/self.Photo.height():
                    self.photo.resize(self.Photo.width(), int(self.photo_image.height()*(self.Photo.width()/self.photo_image.width())))
                else:
                    self.photo.resize(int(self.photo_image.width()*self.Photo.height()/self.photo_image.height()), self.Photo.height())
            else:
                self.photo.resize(self.photo_image.width(),self.photo_image.height())
            self.photo.move(int((self.Photo.width()-self.photo.width())/2),int((self.Photo.height()-self.photo.height())/2))

            if(not hasattr(self,"pix")):
                self.pix = QPixmap(self.photo_path)
                self.photo.setPixmap(self.pix)
                self.photo.setScaledContents(True)
            # self.photo.setStyleSheet("QLabel{border-radius: 8px;border-image: url('" + self.photo_path + "') 0 0 0 0 stretch strectch;background:transparent;}")


    def shot_photo_change(self,shot_photo):

        self.photo_path = shot_photo
        shot_photo_image  = QImage(shot_photo)
        self.photo_image = shot_photo_image
        if shot_photo_image.width() > self.Photo.width() or shot_photo_image.height() > self.Photo.height():

            if shot_photo_image.width()/self.Photo.width()  > shot_photo_image.height()/self.Photo.height():
                self.photo.resize(self.Photo.width(), int(shot_photo_image.height()*(self.Photo.width()/shot_photo_image.width())))
            else:
                self.photo.resize(int(shot_photo_image.width()*self.Photo.height()/shot_photo_image.height()), self.Photo.height())
        else:
            if("svg" in shot_photo.split(".")):
                if(self.Photo.width()/shot_photo_image.width() < self.Photo.height()/shot_photo_image.height()):
                    self.photo.resize(int(shot_photo_image.width()*self.Photo.width()/shot_photo_image.width()),int(shot_photo_image.height()*self.Photo.width()/shot_photo_image.width()))
                else:
                    self.photo.resize(int(shot_photo_image.width() * self.Photo.height() / shot_photo_image.height()),
                                      int(shot_photo_image.height() * self.Photo.height() / shot_photo_image.height()))
            else:
                self.photo.resize(shot_photo_image.width(),shot_photo_image.height())
        self.photo.move(int((self.Photo.width()-self.photo.width())/2),int((self.Photo.height()-self.photo.height())/2))

        self.pix = QPixmap(self.photo_path)
        self.photo.setPixmap(self.pix)
        self.photo.setScaledContents(True)
        # self.photo.setStyleSheet("QLabel{border-radius: 8px;border-image: url('" + shot_photo + "') 0 0 0 0 stretch strectch;background:transparent;}")














import os
import pwd
from PyQt5.QtGui import *
from PyQt5.QtCore import *
import dbus.service
import sys
from models.globals import Globals
from utils.log_manager import LogManager
import getpass
import uuid
import json
from xdg import BaseDirectory as xdg
import configparser
import gettext
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

Specials = ["\"%c\"", "%f","%F","%u","%U","%d","%D","%n","%N","%i","%c","%k","%v","%m","%M", "-caption", "/bin/sh", "sh", "-c", "STARTED_FROM_MENU=yes"]

#鼠标状态
BUTTON_LEAVE = 0
BUTTON_ENTER = 1
BUTTON_PRESSED = 2
BUTTON_DISABLE = 3

###############################安装的目录####################

UBUNTUKYLIN_RES_PATH = (os.path.abspath(os.path.curdir) + "/res/")
UBUNTUKYLIN_DATA_PATH = (os.path.abspath(os.path.curdir) + "/data/")
CACHE_DIR = xdg.xdg_cache_home
UKSC_CACHE_DIR = os.path.join(xdg.xdg_cache_home, "uksc")
if not os.path.exists(UKSC_CACHE_DIR):
        try:
            os.makedirs(UKSC_CACHE_DIR)
        except OSError as e:
            pass
            # if e.errno == errno.EEXIST:
            #     # it seems that another process has already created this
            #     # directory in the meantime, that's ok
            #     pass
            # else:
            #     # the error is due to something else, so we want to raise it
            #     raise
else:
    if(os.path.isdir(UKSC_CACHE_DIR) == False):
        try:
            os.remove(UKSC_CACHE_DIR)
            os.makedirs(UKSC_CACHE_DIR)
        except OSError as e:
            pass

UKSC_CACHE_DIR_LOG = os.path.join(UKSC_CACHE_DIR,"log")
if not os.path.exists(UKSC_CACHE_DIR_LOG):
        try:
            os.makedirs(UKSC_CACHE_DIR_LOG)
        except OSError as e:
            pass

#日志文件
ERR_LOG = os.path.join(UKSC_CACHE_DIR_LOG,"kylin-software-center-error.log")
INFO_LOG = os.path.join(UKSC_CACHE_DIR_LOG,"kylin-software-center-info.log")
DEBUG_LOG = os.path.join(UKSC_CACHE_DIR_LOG,"kylin-software-center-debug.log")
Globals.LOG_INFO = LogManager(INFO_LOG, level='info')
Globals.LOG_ERROR = LogManager(ERR_LOG, level='error')

#图片缓存路径
UBUNTUKYLIN_CACHE_ICON_PATH = os.path.join(UKSC_CACHE_DIR, "icons/")
UBUNTUKYLIN_CACHE_SETADS_PATH =os.path.join(UKSC_CACHE_DIR, "ads/")
UBUNTUKYLIN_CACHE_COVERIMAGE_PATH = os.path.join(UKSC_CACHE_DIR,"coverimage/")
UBUNTUKYLIN_CACHE_NEW_ARRIVALS_PATH = os.path.join(UKSC_CACHE_DIR,"new_arrivals/")
UBUNTUKYLIN_CACHE_PREVIEWIMAGE_PATH = os.path.join(UKSC_CACHE_DIR,"previewImage/")

UBUNTUKYLIN_RES_ICON_PATH = UBUNTUKYLIN_DATA_PATH + "icons/"
UBUNTUKYLIN_RES_COVERIMAGE_PATH = UBUNTUKYLIN_DATA_PATH + "coverimage/"
UBUNTUKYLIN_RES_PREVIEWIMAGE_PATH = UBUNTUKYLIN_DATA_PATH + "previewImage/"
UBUNTUKYLIN_RES_NEW_ARRIVALS_PATH = UBUNTUKYLIN_DATA_PATH + "new_arrivals/"
UBUNTUKYLIN_RES_AD_PATH = UBUNTUKYLIN_DATA_PATH + "ads/"

SVG_ICON_PATH = "/usr/share/icons/hicolor/scalable/apps/"
ICON_48_PATH = "/usr/share/icons/hicolor/48x48/"
ICON_64_PATH = "/usr/share/icons/hicolor/64x64/"
ICON_128_PATH = "/usr/share/icons/hicolor/128x128/"

#商业版系统默认图标路径
KYLIN_SYSTEM_ICON_24_PATH = "/usr/share/icons/kylin-icon-theme/24x24/apps/"
KYLIN_SYSTEM_ICON_48_PATH = "/usr/share/icons/kylin-icon-theme/48x48/apps/"
KYLIN_SYSTEM_ICON_64_PATH = "/usr/share/icons/kylin-icon-theme/64x64/apps/"
KYLIN_SYSTEM_ICON_128_PATH = "/usr/share/icons/kylin-icon-theme/128x128/apps/"
KYLIN_SVG_ICON_PATH = "/usr/share/icons/kylin-icon-theme/scalable/apps/"
#社区版系统默认图标路径
UK_SYSTEM_ICON_24_PATH = "/usr/share/icons/ukui-icon-theme-default/24x24/apps/"
UK_SYSTEM_ICON_48_PATH = "/usr/share/icons/ukui-icon-theme-default/48x48/apps/"
UK_SYSTEM_ICON_64_PATH = "/usr/share/icons/ukui-icon-theme-default/64x64/apps/"
UK_SYSTEM_ICON_128_PATH = "/usr/share/icons/ukui-icon-theme-default/128x128/apps/"
UK_SVG_ICON_PATH = "/usr/share/icons/ukui-icon-theme-default/scalable/apps/"

UBUNTUKYLIN_CACHE_SETSCREENSHOTS_PATH=os.path.join(UKSC_CACHE_DIR, "previewImage/")
if(os.path.exists(UBUNTUKYLIN_CACHE_SETSCREENSHOTS_PATH)):
    pass
else:
    if(os.path.exists(UKSC_CACHE_DIR)):
        os.makedirs(UBUNTUKYLIN_CACHE_SETSCREENSHOTS_PATH)
    else:
        os.makedirs(UKSC_CACHE_DIR)
        os.makedirs(UBUNTUKYLIN_CACHE_SETSCREENSHOTS_PATH)

UBUNTUKYLIN_RES_SCREENSHOT_PATH = os.path.join("/usr/share/kylin-software-center/data/", "previewImage/")
UBUNTUKYLIN_USER_TOKEN_PATH = os.path.join(CACHE_DIR, "kylinId/")

UBUNTUKYLIN_HTTP_WIN_RES_PATH = os.path.join(UKSC_CACHE_DIR,"uk-win/")
UBUNTUKYLIN_RES_WIN_PATH = UBUNTUKYLIN_DATA_PATH + "winicons/"

WGET_DOWN_PATH = os.path.join(os.path.expandvars('$HOME'), ".cache/uksc/wget_down")
STORE_DEB_PATH = os.path.join(os.path.expandvars('$HOME'), ".cache/uksc/wget_down/deb")

KWIN_CONF_PATH = os.path.join(os.path.expandvars('$HOME'), ".config/ukui-kwinrc")
KYLIN_ID_CONF_PATH = os.path.join(os.path.expandvars('$HOME'), ".config/ukui/kylin-id.conf")

CONF_PATH =UKSC_CACHE_DIR + "/kylin-software-center.conf"
if( not os.path.exists(CONF_PATH)):
    os.system("cp /usr/share/kylin-software-center/kylin-software-center.conf "+UKSC_CACHE_DIR)
fd = open(CONF_PATH,"r")
re = fd.readlines()
theme = False
update = False
for i in re:
    if("[theme-color]" in i):
        theme = True
    if("[update-data]" in i):
        update = True
if(theme == False or update == False):
    os.popen("cp /usr/share/kylin-software-center/kylin-software-center.conf " + UKSC_CACHE_DIR)


# 安卓兼容源自动判断
kydroid_source = {
    "V10":{
        "kydroid2": {
            "amd64":"http://archive.kylinos.cn/kylin/kydroid/2/x86/",
            "arm64":"http://archive.kylinos.cn/kylin/kydroid/2/arm64/"
        },
        "kydroid3": {
            # "amd64":"http://archive.kylinos.cn/kylin/kydroid/3/x86/",
            "arm64":"http://archive.kylinos.cn/kylin/kydroid/3/arm64/",
            "kunpeng":"http://archive.kylinos.cn/kylin/kydroid/3/kunpeng/",
            "other":"http://archive.kylinos.cn/kylin/kydroid/3/other/"  #709和景嘉微显卡源
        },
        "kydroid4": {
            "amd64":"http://archive.kylinos.cn/kylin/kydroid/4/x86/",
        },
        "kmre2": {
            # "amd64":"http://archive.kylinos.cn/kylin/kydroid/3/x86/",
            "arm64": "http://archive.kylinos.cn/kylin/kmre/2/arm64/",
            "kunpeng": "http://archive.kylinos.cn/kylin/kmre/2/kunpeng/",
            "other": "http://archive.kylinos.cn/kylin/kmre/2/other/"  # 709和景嘉微显卡源
        }
    },
    "V101":{
        "kydroid3": {
            # "amd64":"http://archive.kylinos.cn/kylin/kydroid/3/x86/",
            "arm64":"http://archive.kylinos.cn/kylin/kydroid/3/101/arm64/",
            "kunpeng":"http://archive.kylinos.cn/kylin/kydroid/3/101/kunpeng/",
            "other":"http://archive.kylinos.cn/kylin/kydroid/3/101/other/",  #709和景嘉微显卡源
            "kyrin990":"http://archive.kylinos.cn/kylin/kydroid/3/101/kyrin990/"
        },
        "kmre2": {
            # "amd64":"http://archive.kylinos.cn/kylin/kydroid/3/x86/",
            "arm64": "http://archive.kylinos.cn/kylin/kmre/2/101/arm64/",
            "kunpeng": "http://archive.kylinos.cn/kylin/kmre/2/101/kunpeng/",
            "other": "http://archive.kylinos.cn/kylin/kmre/2/101/other/"  # 709和景嘉微显卡源
        }
    }
}

#判断和兼容安卓兼容版本
if(os.path.isfile('/usr/share/kydroid/kydroid.conf') and os.path.exists('/usr/lib/libkydroidrequest.so')): # kydroid3和之后版本
    KYDROID_VERSION = "kydroid"
    KYDROID_VERSION_D = "Kydroid"
    KYDROID_CONF_PATH = "/usr/share/kydroid/kydroid.conf"
elif(os.path.isfile('/usr/share/kmre/kmre.conf') and os.path.exists('/usr/lib/libkmre.so')):
    KYDROID_VERSION = "kmre"
    KYDROID_VERSION_D = "Kmre"
    KYDROID_CONF_PATH = "/usr/share/kmre/kmre.conf"
else:
    KYDROID_VERSION = "kydroid2"
    KYDROID_VERSION_D = "Kydroid2"
    KYDROID_CONF_PATH = "/usr/share/kydroid2/kydroid2.conf"

system_version =  ""
CPU = ""
GPU = ""
try:
    try:
        bus = dbus.SessionBus()
    except:
        sys.exit(0)
    if (os.path.exists("/usr/share/kmre/kmre.conf")):
        proxy_obj = bus.get_object('cn.kylinos.Kmre.Manager', '/cn/kylinos/Kmre/Manager')
        iface = dbus.Interface(proxy_obj, 'cn.kylinos.Kmre.Manager')
    else:
        proxy_obj = bus.get_object('com.kylin.kydroid', '/com/kylin/kydroid')
        iface = dbus.Interface(proxy_obj, 'com.kylin.kydroid')
    dest = iface.getDisplayInformation()
    adev=json.loads(dest)
    # arch = os.popen("dpkg --print-architecture").readline().splitlines()[0]
    kydroid_config = configparser.ConfigParser()
    if "gpu_vendor" in adev.keys() and "gpu_model" in adev.keys():
        GPU = adev["gpu_vendor"]
        if (GPU == "JJM" and "gpu_model" in adev):
            if (adev["gpu_model"] == "JJM_7200"):
                GPU = "JJM"
            else:
                GPU = ""
        elif (GPU == "GP101" and "gpu_model" in adev):
            if (adev["gpu_model"] == "GP101_0001"):
                GPU = "GP101"
            else:
                GPU = ""
        if("cpu_type" in adev.keys()) :
            CPU = adev["cpu_type"]
    if "gpu_vendor" in adev:
        if(GPU == ""):
            GPU = adev["gpu_vendor"]
        # if arch == "amd64":
        #     pass
        # else:
        #     if adev["gpu_vendor"] == "AMD" or adev["gpu_vendor"] == "NVIDIA":
        #         arch = "arm64"
        #     elif adev["gpu_vendor"] == "GP101" or adev["gpu_vendor"] == "JJM" or adev["gpu_vendor"] == "MALI" or adev[
        #         "gpu_vendor"] == "VIRTUAL" or adev["gpu_vendor"] == "UNKNOWN" or adev["gpu_vendor"] == "ZC716":
        #         arch = "other"

        Globals.ADEV = adev["gpu_vendor"]
    # if ((os.popen("lspci -n|awk '{print $3}' |grep '0709:'").read() != '') or (
    #         os.popen("cat /proc/fb |grep -i MWV206").read() != '')):  # 709和景嘉微特殊处理
    #     arch = "other"
    # elif (os.popen("lscpu|grep -i kunpeng").read() != ''):
    #     arch = "kunpeng"
    # elif (CPU == "HUAWEI_KIRIN990"):
    #     arch = "kirin990"
    # read_system_version = os.popen("lsb_release -d").readline()
    # res_list = read_system_version.split(":")
    # res_list1 = res_list[1].strip("\t").strip("\n").strip(" ")
    # res_list2 = res_list1.split("Kylin")
    # res = res_list2[1].strip(" ").strip("\n").strip("\t")
    # system_version = res

    kydroid_config.read(KYDROID_CONF_PATH)
    kydroid_version = kydroid_config['image']['repo']
    #传参：系统版本、安卓版本、架构给服务器获取返回
    # SYSTEM_VERSION = system_version
    GET_KYDROID_VERSION = kydroid_version
    # KYDROID_ARCH = arch
    # KYDROID_SOURCE_SERVER = kydroid_source[system_version][kydroid_version][arch]
except:
    KYDROID_SOURCE_SERVER = ""
    # SYSTEM_VERSION = "1"
    GET_KYDROID_VERSION = "1"
    # KYDROID_ARCH = "1"
    # caoliang
    UBUNTUKYLIN_ARCH = "all"

SOFTWARE_BLACK ="/home/"+getpass.getuser()+"/"+".config/kylin-software-center-security-config.json"

def software_center_noshow():
    try:
        blacklist=[]
        file =open(SOFTWARE_BLACK,"r")
        dest=json.load(file)
        ntdit=dest['ukui-kylin-software-center']['blacklist']
        for i in ntdit:
            blacklist.append(i['pkgname'])
        return blacklist
    except:
        return []

Globals.BLACKLIST = software_center_noshow()

###############################缓存路径###############################

#KYDROID_DOWNLOAD_PATH = "/var/lib/kydroid/kydroid2-1000-kylin/data/local/tmp"
if(os.path.exists("/usr/share/kmre/kmre.conf")):
    KYDROID_DOWNLOAD_PATH = "/var/lib/kmre/" + KYDROID_VERSION + "-" + str(os.getuid()) + "-" + str(pwd.getpwuid(os.getuid())[0]) + "/data/local/tmp/"
    KYDROID_STARTAPP_ENV = "/usr/bin/startapp start_kmre"
else:
    KYDROID_DOWNLOAD_PATH = "/var/lib/kydroid/" + KYDROID_VERSION + "-" + str(os.getuid()) + "-" + str(pwd.getpwuid(os.getuid())[0]) + "/data/local/tmp/"
    KYDROID_STARTAPP_ENV = "/usr/bin/startapp start_kydroid"

KYDROID_INSTALLED_FILENAME = KYDROID_DOWNLOAD_PATH +"/installed.json"





########################################网址类###########################

UBUNTUKYLIN_SERVICE_PATH = "com.kylin.softwarecenter"
UBUNTUKYLIN_INTERFACE_PATH = "com.kylin.softwarecenter"

UBUNTUKYLIN_SERVER = "http://service.ubuntukylin.com:8001/uksc/"
UK_APP_ICON_URL = ("http://service.ubuntukylin.com:8001/"
    "media/%(pkgname)s")

# screenshot handling
SCREENSHOT_THUMB_URL = ("http://screenshots.ubuntu.com/"
    "thumbnail-with-version/%(pkgname)s/%(version)s")
SCREENSHOT_LARGE_URL = ("http://screenshots.ubuntu.com/"
    "screenshot-with-version/%(pkgname)s/%(version)s")


RESOURCE_SERVER="http://archive.kylinos.cn/kylin/resources/screenshots/"

#####################################样式###################################
ITEM_LABEL_STYLE = ("QLabel{background-image:url(%s);background-color:transparent;}")


#####################################类###################################
class Signals:
    # from models.application import Application
    init_models_ready = pyqtSignal(str,str)

    download_ranking_ready = pyqtSignal(list)
    hot_ready = pyqtSignal(list,list)
    get_ad_ready = pyqtSignal(list,list)
    get_new_arrivals_ready = pyqtSignal(list,list)
    get_android_recommend_ready = pyqtSignal(list,list)
    get_win_replace_ready = pyqtSignal(list,list)
    recommend_ready = pyqtSignal(list,bool,bool)
################
    app_reviews_ready = pyqtSignal(int,list,str)

    get_reviews = pyqtSignal(str)

    # caoliang login signals
    ui_login = pyqtSignal(list)
    get_ui_login_over = pyqtSignal(list)
    get_ui_login_over_fail = pyqtSignal(bool)
    ui_login_success = pyqtSignal()
    save_usrname_password = pyqtSignal(str,str,str)

    #zou
    ui_login_mouse_current_position = pyqtSignal(int)
    #zouzou
    ui_login_get_image_code = pyqtSignal(dict)
    ui_login_image_code_move = pyqtSignal(int)
    #zou手机号
    ui_login_phone_number = pyqtSignal(list)


class AppMsg:
    ID = 0
    APPNAME = ""
    DISPLAY_NAME = ""
    DISPLAY_NAME_CN = ""
    CATEGORIES = ""
    CATEGORIES_CN = ""
    DESCRIPTION = ""
    RATING_AVG = ""
    RATING_TOTAL = 0
    REVIEW_TOTAL = 0
    DOWNLOAD_TOTAL = ""
    VERSION = ""
    IS_FROM_DB = True
    IS_INSTALLED = False
    IS_UPGRADABLE = False
    IS_KYDROID = False
    FILE_PATH = ""
    SIZE = ""
    RUN = None

AptActionMsg = {
    "install_deps":_("Install dependencies"),
    "install_debfile":_("Install local package"),
    "install":_("Software Installation"),
    "remove":_("Software uninstall"),
    "upgrade":_("Software upgrade"),
    "update":_("Source update"),
    "update_first":_("Source initialization")
}

AptProcessMsg = {
    "apt_start":_("Start..."),
    "apt_finish":_("Perfection!"),
    "apt_error":_("Failure!"),
    "apt_pulse":_("Processing"),
    "down_start":_("Download begins"),
    "down_stop":_("Download stopped"),
    "down_done":_("Download completed"),
    "down_fail":_("download failed"),
    "down_fetch":_("Single download completed"),
    "down_pulse":_("Download in progress"),
    "down_cancel":_("Download canceled"),
}





# application actions, this should sync with definition in apt_dbus_service
class AppActions:
    INSTALLDEPS = "install_deps"
    INSTALLDEBFILE = "install_debfile"
    DOWNLOADAPK = "download_apk"
    INSTALL = "install"
    REMOVE = "remove"
    UPGRADE = "upgrade"
    CANCEL = "cancel"
    APPLY = "apply_changes"
    PURCHASE = "purchase"
    UPDATE = "update"
    UPDATE_FIRST = "update_first"
    ADD_SOURCE = "add_source"
    REMOVE_SOURCE = "remove_source"
    GET_SOURCES = "get_sources"
    USECDROM = "usecdrom"
    FIND_UP_SERVER = "find_up_server"

class PageStates:
    (
     HOMEPAGE,
     MYPAGE,
     DOWNLOADINGPAGE,
     SEARCHPAGE,
     WINPAGE,
     CAREGORYPAGE,
     KYDROID,
     HISTORY,
     APPDETAIL,
     MOREPAGE,
     DRIVE
     ) = list(range(11))

class ClickCard:
    (
        NEWARRIVALS,
        AD,
        INSIDECARD,
        HOT
    ) = list(range(4))

# pkg action state constants
class PkgStates:
    (
    # current
    INSTALLED,
    UNINSTALLED,
    UPGRADABLE,
    REINSTALLABLE,
    # progress
    INSTALLING,
    REMOVING,
    UPGRADING,
    ENABLING_SOURCE,
    INSTALLING_PURCHASED,
    # special
    NEEDS_SOURCE,
    NEEDS_PURCHASE,
    PURCHASED_BUT_REPO_MUST_BE_ENABLED,
    ERROR,
    FORCE_VERSION,
    # the package is not found in the DB or cache
    NOT_FOUND,
    # its purchased but not found for the current series
    PURCHASED_BUT_NOT_AVAILABLE_FOR_SERIES,
    # this *needs* to be last (for test_appdetails.py) and means
    # something went wrong and we don't have a state for this PKG
    UNKNOWN,
    DOWNLOADING,
    PUASE,
    RUN,
    INSTALL,
    UPDATE,
    UNINSTALL,
    NORUN,
    NOTHING,
    REMOVED,
    ) = list(range(26))

#软件状态
DOWNLOADING = 1
PUASE = 2
INSTALLING = 3
OTHER = 4


def setLongTextToElideFormat(label, text):
    if text[len(text) - 1] == '\n':
        text = text.rstrip()
    metrics = QFontMetrics(label.font())
    elidedText = metrics.elidedText(text, Qt.ElideRight, label.maximumWidth())
    label.setText(elidedText)
    return elidedText

DB_ITEM = {"action":"","appname":""}

get_app_info = {"name":"","displayname":"","displayname_cn":"","categories":"","summary":"","description":"","rating_avg":0,"rating_total":0,"review_total":0,"download_total":0,"from_db":True}

category_dict = {"id":0,"name":"","displayname":""}

PKG_NAME = {}
ARCH = ""
SYSTEM_VERSION = ""
